require 'json'
require 'structure_validator/version'

module StructureValidator
  class UnsupportedDataType < RuntimeError; end

  def self.validate(object, expected_structure)
    check_required_keys(object, expected_structure) && check_type_for_each_key(object, expected_structure)
  end

  private

  def self.check_required_keys(object, expected_structure)
    (expected_structure.keys - object.keys).empty?
  end

  def self.check_type_for_each_key(object, expected_structure)
    expected_structure.delete_if do |key, type|
      if type.is_a?(Hash) && object[key].is_a?(Hash)
        validate(object[key], type)
      else
        test_type_for(object[key], type)
      end
    end.empty?
  end

  def self.test_type_for(value, type)
    case type.to_sym
    when :string
      value.is_a? String
    when :integer
      value.is_a? Integer
    when :json
      begin
        !!JSON.parse(value)
      rescue JSON::ParserError
        false
      end
    else
      raise UnsupportedDataType.new("Unexpected type: #{type}")
    end
  end
end
